#ifndef __SCANNER_SERVER_H
#define __SCANNER_SERVER_H

#ifndef __SMALL_SOCKET_H
#include "small_socket.h"
#endif

#ifndef __FP_H
#include "fp.h"
#endif

#ifndef __FP_RFC_H
#include "fp_rfc.h"
#endif

#ifndef __FP_RETURN_H
#include "fp_return.h"
#endif

#ifndef __FP_CONF_H
#include "fp_conf.h"
#endif

#ifndef __MFILE_H
#include "mfile.h"
#endif

#ifndef __SMTPMAP_H
#include "smtpmap.h"
#endif

#ifndef __COMMANDS_H
#include "commands.h"
#endif

class scanner_server
{
private:

protected:
   /**
    * List of hostnames that will be stripped of server answers. Should
    * contain localhost, local hostname, local IP(s), remote IP(s), remote
    * Hostname, eventually aliases
    */
    dllist<mstring> hostname_list;

    dllist<mstring> strip_list;

   /**
    * @name Fingerprint Reading Functions
    * These Functions are used to read the fingerprints from a specified file
    * into the derived classes fingerprint lists.
    */

    //@{
   /**
    * This function is called whenever a full Fingerprint has been build. The derived class may understand different types of fingerprints and save them within their structures.
    * @param fp& n_fp Reference to the newly generated fingerprint.
    * @param fp_type f_type Type identifier of the Fingerprint type
    * @return true if the derived class has accepted the fingerprint as valid.
    */
    virtual bool add_fp( fp& n_fp, fp_type f_type)=0;
    bool load_fp (const mstring& file, const mstring& identifier, fp_type fpt, bool check_leng);
    //@}
    //
   /**
    * Every scanner must have a socket, so hold it here
    */
    small_socket& sock;
    
   /**
    * Flag, if we should be polite when reading fps from file
    */
    bool polite;

    uint32 cmd_number;

    /**
     * @name Connection Options
     * These options choose how the connection will be handled and initialized
     * over the scan.
     */
    //@{ 
    /**
     * Also the target host is for all servers in common
     */
    mstring target_host;

    mstring username;
    
    mstring creative_username;
    mstring creative_hostname;
    
    mstring password;

    virtual bool scan_add_to_fp( const mstring&  answer ) = 0;

    /**
     * Also the target port is for all servers in common
     */
    uint32 target_port;

    /**
     * The debuglevel, that says, what to output
     */
    debug_level debuglevel;

    /**
     * Connection timeout.
     */
    uint32 conn_timeout;
    
    /**
     * 
     */
    uint32 recv_timeout;
    //@}

    /**
     * @name Internal Helper Functions
     * These functions help to do often used things easily, like readingfiles
     * or answers from the files.
     */
    //@{

    /**
     * Gets the next line of a file, but ignores comment lines, and also strips comments out
     * @param mfile& datei File from that the next line will be read
     * @return mstring String that contains the next valid non-comment line, without additional comments
     */
    mstring get_next_line ( mfile& datei);

    /**
     * Gets the whole answer from the network, that is sent as reaction to a command. 
     * @param none
     * @return the complete (maybe multilined) answer from the server
     */
    mstring get_rfc_answer ( void );
    //@}

    void printerror( const mstring & error, uint32 line );
    
    sint32 gen_fp_data (fp * cfp, mstring & fp);
    
    uint32 check_lenge (uint32 lenge, uint32 fps, uint32 lines ,bool polite);
    
    bool scan_step( );
    
    virtual bool scan_init( void ) = 0;
    
    virtual bool scan_finished( void ) = 0;

    virtual bool scan_reconnected( long number ) = 0;

    scan_command * cmd_list;

    scan_command * actual_command;
    
    uint32 command_count;
    
    virtual bool scan_special( long number ) = 0;
    /// Gets a Singleline Answer from the server.
    mstring get_answer_single( void );
    /// Gets the first line of a multiline answer (or only a line if -ERR line)
    mstring get_answer_multi( void );

    uint32 recon_count;

    uint32 max_recon;
    


public:
    virtual bool do_scan( const mstring& username, const mstring& password ); 
    /**
     * Constructs the actual scanner base.
     * @param small_socket& sk Socket that will be used to connect (should not be connected)
     * @param int debug Debuglevel to be used in this instantiation.
     * @param bool pol If to be polite when reading input
     * @param long ct Connection Timeout in milliseconds
     * @param long rt Receive Timeout in milliseconds
     */
    scanner_server( small_socket& sk, debug_level debug, bool pol, long ct, long rt) : sock(sk),polite(pol),target_host("127.0.0.1"), target_port(25),debuglevel(debug), conn_timeout(ct), recv_timeout(rt),max_recon(5){};
    
    /**
     * Actually the data will be held in derived classes, so nothing to be done here
     */
    virtual ~scanner_server() { }

    /**
     * Initializes the scanner, needs to be done before scanning process (reads fingerprints)
     * @return bool If the init was succesfully. If not, the scanner should not be used
     */
    virtual bool initialize( void ) = 0;

    /**
     * @return mstring that will contain the protocoll name, i.e. POP3 or SMTP or FTP or...
     */
    virtual mstring protocol( void ) { return "Virtual Protocol"; }

    /**
     * @return mstring Short information, what scanner this might be (fast scanning)
     */
    virtual mstring is_server( void ) = 0;

    /**
     * Ends the scanning process, and calculates the matching values of the fingerprints
     */
    virtual void finalize( void ) = 0;

    /**
     * @return list of list of fingerprints. Defined list, that helds the error mathces of the fps
     */
    virtual dllist < dllist < fp > > get_fps( void ) =0;

    /**
     * @return list of mstrings that helds the type information of the fingerprint lists
     */
    virtual dllist< mstring > get_descs( void ) =0;

    /**
     * That does the actual scan
     * @return bool if something went wrong.
     */


    /**
     * Sets the target and port, that will be used for the scanning process, so the scanner can be reused
     * for multiple scans.
     * @param mstring target Target host to be scanned, preffereble as an IP
     * @param int port Port to be used for this scan, depends on derived service type
     */
    inline virtual void set_target( mstring target, int port) { target_host = target; target_port = port; }
};

#endif
